#!/usr/bin/perl -T
############################################################
############################################################
###                                                      ###
###   Originally designed by Soeren Koch                 ###
###                                                      ###
###   Impelemented by Soeren Koch                        ###
###                                                      ###
###   Copyright by Soeren Koch January 2011              ###
###                                                      ###
############################################################
############################################################

use strict;
use warnings;
use RFC::Main;
use RFC::Rig;
use RFC::RFCCGI qw(:cgi :html);
use RFC::Header;
use CGI qw(:standard :netscape);

my $r = param('rig');
unless ($r)
{
    print header();
    print "No rig specified!\n";
    exit;
}
my $rig = RFC::Rig->new($r);
my @devs = $rig->list_devices_control('PSU');
my %devlabels = ();
foreach my $name (@devs)
{
    my $dev = $rig->get_device('PSU',$name);
    $devlabels{$name} = $dev->UIname();
}
foreach my $n ($rig->list_devices_control('analog'))
{
    my $dev = $rig->get_device("analog",$n);
    $dev->init();
    next unless ($dev->mode() && $dev->mode() =~ /A/i);
    push @devs, $n.' (analog)';
    $devlabels{$n.' (analog)'} = $dev->UIname().' (analog)';
}

if (!defined(param))
{
    &print_header("Please log in to the $name");
    print '<center>';
    RFC::RFCCGI::login();
    print '</center>';
    &print_end;
}
else
{
    #### husk at aendre accessparameter til det relevante for scriptet!! ####
    if (&login_ok())
    {
        &print_header("Set current for rig ".$rig->name);
        if ($ENV{'HTTP_USER_AGENT'} =~ /MSIE/)
        {
	    print RFC_start_html(-title=>"Set current for rig ".$rig->name, -style=>{src=>'/celltest/RFCcontrol.css'},-onBlur=>'initiateSelfClosing()');
        }
        else
        {
	    print RFC_start_html(-title=>"Set current for rig ".$rig->name, -style=>{src=>'/celltest/RFCcontrol.css'},-onBlur=>'self.close()');
	}
	print '<center>';
	print '<table border=0 cellspacing=5 cellpadding=0 style="width:100%"><tr height=1>';
	print '<td width="5%"></td>';
	print '<td width="30%"></td>';
	print '<td width="25%"></td>';
	print '<td width="5%"></td>';
	print '<td width="30%"></td>';
	print '<td width="5%"></td>';
	print '</tr>';
	print start_form(-name=>'sofc');
	print '<tr><td colspan=6 class=heading> Set current for rig ',$rig->name,'</td></tr>';
	print '<tr><td></td><th>';
	my ($t,$j) = ('','');
	if (@devs > 1)
	{
	    print 'Select controler';
	    print popup_menu(-name=>'device',-values=>['',sort @devs],-labels=>\%devlabels,-default=>'',-class=>'textfield',-force=>'1',-onchange=>'set_default()');	    
	    $j = "    var element = document.sofc.device;\n";
	    $j .= "    opener.document.PSU.device.value = element[element.selectedIndex].value;\n";
	}
	else
	{
	    if ($devs[0] =~ /(\w+)\s\(analog\)/)
	    {
		my $d = $rig->get_device('analog',$1);
		$t = $d->get_I();		
	    }
	    else
	    {
		my $d = $rig->get_device('PSU',$devs[0]);
		$t = $d->get_I();
	    }
	    $j = "    opener.document.PSU.device.value = document.sofc.device.value;\n";
	    print hidden(-name=>'device',-value=>$devs[0],-force=>1);
	}
	print '</th><td colspan=2>';
	print '</td><td rowspan=2>';
	print button(-name=>'test',-value=>'Ok',-class=>'s3',-OnClick=>'set_curr()');
	print '</td></tr>',"\n";
	print '<tr><td></td><th> Current: </th><td>';
	print textfield(-name=>'current',-value=>$t,-force=>'1',-class=>'textfield');
	print '</td><th>A</th></tr>',"\n";
	print '<tr><td></td><td colspan=4>';
	print button(-name=>'ocv',-value=>'Set OCV',-OnClick=>'set_OCV()',-class=>'s2');
	print '</td></tr>';
        if (@devs == 1)
        {
	    my $d;
	    if ($devs[0] =~ /(\w+)\s\(analog\)/)
	    {
		$d = $rig->get_device('analog',$1);
	    }
	    else
	    {
		$d = $rig->get_device('PSU',$devs[0]);
	    }	  
	    if ($d && $d->has_ramp)
	    {
		print '<tr><td></td><th>Ramprate:</th><td>';
		print textfield(-name=>'ramprate',-value=>$r,-force=>'1',-class=>'textfield');
		print '</td><th>A/hour</th></tr>',"\n";         
	    }
	    else
	    {
		print '<tr><td></td><th id="ramp_text"></th><td>';
		print textfield(-id=>'ramprate',-name=>'ramprate',-value=>'',-force=>'1',-class=>'hidden');
		print '</td><th id="unit3"></th></tr>',"\n";		
	    }
        }
        else
        {
            print '<tr><td></td><th id="ramp_text"></th><td>';
            print textfield(-id=>'ramprate',-name=>'ramprate',-value=>'',-force=>'1',-class=>'hidden');
            print '</td><th id="unit3"></th></tr>',"\n";
        }
	print end_form;
	print <<END;

<script language=javascript>

function set_OCV()
{
$j    
    opener.document.PSU.current.value = 'OCV';
    opener.set_PSU_go();
    self.close();
}

function set_default()
{
    var element = document.sofc.device;
    var id = element.selectedIndex;
    var name = element[id].value;
    document.sofc.current.disabled = true; 
    document.sofc.ramprate.disabled = true;
    var re = /\\(analog\\)/;
    if (re.test(name))
    {
        name = name.replace(/\\s+\\(analog\\)/,"");
	run_ajax('rig_ajax_run.cgi','&action=run_device_function&type=analog&dname='+name+'&function=setpoint',set_default_analog_ajax);
    }
    else
    {
	run_ajax('rig_ajax_run.cgi','&action=run_device_function&type=PSU&dname='+name+'&function=get_I',set_default_ajax);
    }
}


function set_ramprate(res)
{
    var arr = res.split("\\n");
    if (arr[0] == "OK")
    {
        var has_ramp = false;
        if (arr[1] == 1)
        {
            var has_ramp = true;
        }
        var ramp = document.getElementById("ramprate");
        var ramp_text = document.getElementById("ramp_text");
        var unit = 'A';
        var unit3 = document.getElementById("unit3");
        if (has_ramp)
        {
            unit3.innerHTML = unit+"/hour";
            ramp_text.innerHTML = "Ramprate:";
            ramp.value = arr[2];
            ramp.className = "textfield";
        }
        else
        {
            unit3.innerHTML = "";
            ramp_text.innerHTML = "";
            ramp.className = "hidden";
        }
    }
    else
    {
        alert(res);
    }
    document.sofc.current.disabled = false;
    document.sofc.ramprate.disabled = false;
}


function set_default_analog_ajax(res)
{
    var arr = res.split("\\n");
    if (arr[0] == "OK")
    {
        document.sofc.current.value = arr[1];
        document.sofc.current.disabled = false;    
        document.sofc.ramprate.disabled = false;    
    }
    else
    {
        alert(res);
    }
    var element = document.sofc.device;
    var id = element.selectedIndex;
    var name = element[id].value;
    var reg = /\(analog\)/;
    if (reg.test(name))
    {
        var a = name.split(' (');
        run_ajax('rig_ajax_run.cgi','&action=get_ramprate&type=analog&dname='+a[0],set_ramprate);
    }
}

function set_default_ajax(res)
{
    var arr = res.split("\\n");
    if (arr[0] == "OK")
    {
        document.sofc.current.value = arr[1];      
        document.sofc.current.disabled = false;
        document.sofc.ramprate.disabled = false;
    }
    else
    {
        alert(res);
    }
    var element = document.sofc.device;
    var id = element.selectedIndex;
    var name = element[id].value;
    var reg = /\(analog\)/;
    if (reg.test(name))
    {
        var a = name.split(' (');
        run_ajax('rig_ajax_run.cgi','&action=get_ramprate&type=analog&dname='+a[0],set_ramprate);
    }
    else
    {
        run_ajax('rig_ajax_run.cgi','&action=get_ramprate&type=PSU&dname='+name,set_ramprate);
    }
}



function set_curr()
{
$j    
    opener.document.PSU.current.value = document.sofc.current.value;
    opener.document.PSU.ramp.value = document.sofc.ramprate.value;
    opener.set_PSU_go();
    self.close();
}

</script>

END

        print end_html;
    }
    else
    {
        &not_auth;
    }
}

